# Copyright (c) 2024 Gaurav Aggarwal

from importlib.resources import files
from typing import List

PACKAGE_DATA_PATH = 'mlx_hub.data'


def read_packaged_file(file_name: str) -> List[str]:
    """Reads and returns text from packaged file."""
    try:
        with files(PACKAGE_DATA_PATH).joinpath(file_name).open() as file_name:
            lines = file_name.readlines()
            return [line.strip() for line in lines]
    except (FileNotFoundError, IOError):
        print(f"An error occurred while reading the file at path {file_name}.")
        return []


def print_packaged_file(file_name: str):
    """Prints text from packaged file."""
    for line in read_packaged_file(file_name):
        print(line)



# Copyright (c) 2024 Gaurav Aggarwal

import unittest
import mlx_hub

TEST_MODEL = "mlx-community/TinyDolphin-2.8-1.1b-4bit-mlx"
TEST_SEARCH_PHRASE_MANY_RESULTS = "llama"
TEST_SEARCH_PHRASE_FEW_RESULTS = "bert"
TEST_SEARCH_PHRASE_NO_RESULTS = "bad_search_phrase"
TEST_SEARCH_PHRASE_EMPTY = ""


class TestMLXHub(unittest.TestCase):
    def test_search(self):
        result = mlx_hub.search(TEST_SEARCH_PHRASE_MANY_RESULTS)
        self.assertIsNotNone(result)
        self.assertEqual(len(result), mlx_hub.SEARCH_LIMIT)

        for model_id in result:
            self.assertIsNotNone(model_id)
            self.assertGreater(len(model_id), 0)
            self.assertEqual(len(model_id.split("/")), 2)

        result = mlx_hub.search(TEST_SEARCH_PHRASE_FEW_RESULTS)
        self.assertIsNotNone(result)
        self.assertLess(len(result), mlx_hub.SEARCH_LIMIT)

        for model_id in result:
            self.assertIsNotNone(model_id)
            self.assertGreater(len(model_id), 0)
            self.assertEqual(len(model_id.split("/")), 2)

        result = mlx_hub.search(TEST_SEARCH_PHRASE_NO_RESULTS)
        self.assertIsNotNone(result)
        self.assertEqual(len(result), 0)

        result = mlx_hub.search(TEST_SEARCH_PHRASE_EMPTY)
        self.assertIsNotNone(result)
        self.assertEqual(len(result), mlx_hub.SEARCH_LIMIT)

    def test_suggest(self):
        result = mlx_hub.suggest()
        self.assertIsNotNone(result)
        self.assertGreaterEqual(len(result), 0)

        for model_id in result:
            self.assertIsNotNone(model_id)
            self.assertGreater(len(model_id), 0)
            self.assertEqual(len(model_id.split("/")), 2)

    def test_scan(self):
        result = mlx_hub.scan()
        self.assertIsNotNone(result)
        self.assertGreaterEqual(len(result), 0)

        if TEST_MODEL not in result:
            download_result = mlx_hub.download(TEST_MODEL)
            self.assertEqual(download_result, True)
            result = mlx_hub.scan()
            self.assertIn(TEST_MODEL, result)

        for model_id in result:
            self.assertIsNotNone(model_id)
            self.assertGreater(len(model_id), 0)
            self.assertEqual(len(model_id.split("/")), 2)

    def test_download(self):
        scan_result_before_download = mlx_hub.scan()

        if TEST_MODEL in scan_result_before_download:
            delete_result = mlx_hub.delete(TEST_MODEL)
            self.assertEqual(delete_result, True)
            self.assertNotIn(TEST_MODEL, mlx_hub.scan())

        download_result = mlx_hub.download(TEST_MODEL)
        self.assertTrue(download_result)

        scan_result_after = mlx_hub.scan()
        self.assertIn(TEST_MODEL, scan_result_after)

    def test_delete(self):
        scan_results_before_delete = mlx_hub.scan()

        if TEST_MODEL not in scan_results_before_delete:
            download_result = mlx_hub.download(TEST_MODEL)
            self.assertEqual(download_result, True)
            scan_results_after_download = mlx_hub.scan()
            self.assertIn(TEST_MODEL, scan_results_after_download)

        delete_result = mlx_hub.delete(TEST_MODEL)
        self.assertEqual(delete_result, True)

        scan_results_after_delete = mlx_hub.scan()
        self.assertNotIn(TEST_MODEL, scan_results_after_delete)


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2024 Gaurav Aggarwal

from enum import Enum
from typing import List
from huggingface_hub import HfApi, get_token, scan_cache_dir, snapshot_download, CacheNotFound
from huggingface_hub.errors import LocalTokenNotFoundError
import mlx_hub.utils as utils

SUGGESTED_MODELS_FILE = 'suggested_models.txt'

SEARCH_LIBRARY = 'mlx'
SEARCH_FULL = False
SEARCH_LIMIT = 25


class SortBy(Enum):
    LIKES = "likes"
    DOWNLOADS = "downloads"
    CREATED = "created"
    LAST_MODIFIED = "last_modified"


def has_token() -> bool:
    """Checks if there is an active Hugging Face token."""
    try:
        return get_token() is not None
    except LocalTokenNotFoundError:
        return False


def search(search_phrase, search_limit=SEARCH_LIMIT, sort_by=SortBy.DOWNLOADS) -> List[str]:
    """Searches for models using a string that contain complete or partial names on the Hub."""
    hf_api = HfApi()
    models = hf_api.list_models(
        search=search_phrase,
        library=SEARCH_LIBRARY,
        sort=sort_by.value,
        full=SEARCH_FULL,
        limit=search_limit
    )
    return [model.id for model in models]


def suggest() -> List[str]:
    """Reads and returns suggested models from a file."""
    return utils.read_packaged_file(SUGGESTED_MODELS_FILE)


def scan() -> List[str]:
    """Scans the Hugging Face cache directory and returns a list of Models."""
    try:
        hf_cache_info = scan_cache_dir()
        return [model.repo_id for model in hf_cache_info.repos]
    except CacheNotFound:
        return []


def download(model_id: str) -> bool:
    """Downloads a model snapshot by its ID."""
    try:
        return bool(snapshot_download(model_id))
    except Exception as e:
        print(f"An error occurred while downloading the model {model_id}: {e}")
        return False


def delete(model_id: str) -> bool:
    """Deletes a model from the Hugging Face cache by its ID."""
    hf_cache_info = scan_cache_dir()
    for repo in hf_cache_info.repos:
        if model_id == repo.repo_id:
            try:
                for revision in sorted(repo.revisions, key=lambda rev: rev.commit_hash):
                    strategy = hf_cache_info.delete_revisions(revision.commit_hash)
                    strategy.execute()
                return True
            except Exception as e:
                print(f"An error occurred while deleting the model {model_id}: {e}")
                return False
    return False



# Copyright (c) 2024 Gaurav Aggarwal

import argparse
import mlx_hub.core as mlx_hub
import mlx_hub.utils as utils

from enum import Enum

LOGIN_HELP_FILE = "login_help.txt"


class Action(Enum):
    LOGIN = ("login", "", "Add Hugging Face access token", False)
    START = ("start", "", "Start Interactive Mode", False)
    SCAN = ("scan", "", "Scan for downloaded models in the Hugging Face cache")
    SEARCH = ("search", "phrase", "Search for MLX models using a search phrase")
    SUGGEST = ("suggest", "", "Suggest MLX models to download")
    DOWNLOAD = ("download", "model_id", "Download a specific model")
    DELETE = ("delete", "model_id", "Delete a specific model")
    EXIT = ("exit", "", "Exit Interactive Mode")
    HELP = ("help", "", "Show this help message")
    INVALID = ("invalid", "", "Invalid action", False)

    def __new__(cls, value: str, parameter: str, description: str, is_interactive: bool = True):
        obj = object.__new__(cls)
        obj._value_ = value
        obj.option = "--" + value
        obj.parameter = parameter
        obj.description = description
        obj.interactive = is_interactive
        return obj


def print_action_login_help():
    utils.print_packaged_file(LOGIN_HELP_FILE)


def print_action_help():
    """Prints available actions for interactive mode."""
    print("Available Actions:")
    for action in Action:
        if action.interactive:
            print(f"    {action.value:<10} {action.parameter:<10} {action.description}")


def print_string_list(string_list):
    for item in string_list:
        print(item)


def execute_action(action, parameter=None):
    """Executes the given action with optional parameters."""
    print()

    if action == Action.SCAN:
        model_list = mlx_hub.scan()
        if len(model_list) == 0:
            print("No models found!")
        else:
            print(f"{len(model_list)} models found: ")
        print_string_list(model_list)

    elif action == Action.SEARCH:
        if parameter:
            models_list = mlx_hub.search(parameter)
            count = len(models_list)
            if count == 0:
                print("No models found!")
            elif count == mlx_hub.SEARCH_LIMIT:
                print(f"Showing top {mlx_hub.SEARCH_LIMIT} results:")
            else:
                print(f"{count} models found:")
            print_string_list(models_list)
        else:
            print("Please provide a search phrase.")

    elif action == Action.SUGGEST:
        print("Suggested models:")
        print_string_list(mlx_hub.suggest())

    elif action == Action.DOWNLOAD:
        if parameter:
            print(f"Downloading model: {parameter}")
            if mlx_hub.download(parameter):
                print("Model downloaded successfully.")
            else:
                print("Model not found.")
        else:
            print("Please provide a Model ID.")

    elif action == Action.DELETE:
        if parameter:
            print(f"Deleting model: {parameter}")
            if mlx_hub.delete(parameter):
                print("Model deleted successfully.")
            else:
                print("Model not found.")
        else:
            print("Please provide a Model ID.")

    elif action == Action.START:
        print("Starting interactive mode.")
        execute_action(Action.HELP)
        start_interactive_mode()

    elif action == Action.EXIT:
        print("Goodbye!")
        return False

    elif action == Action.LOGIN:
        print_action_login_help()

    elif action == Action.HELP:
        print_action_help()

    else:
        print("Invalid action!\n")
        print_action_help()

    print()
    return True


def start_interactive_mode():
    """Starts the interactive mode, prompting the user for actions."""

    running = True
    while running:
        # Get user input
        user_input = input("Enter Action > ").strip().split(sep=" ", maxsplit=1)
        if not user_input:
            continue

        # Validate action
        try:
            action = Action(user_input[0])
            if not action.interactive:
                action = Action.INVALID

        except ValueError:
            action = Action.INVALID

        # Execute action
        parameter = user_input[1] if len(user_input) > 1 else None
        running = execute_action(action, parameter)


def main():
    """Main function to parse arguments and execute corresponding actions."""
    parser = argparse.ArgumentParser(description='MLX-Hub CLI')
    parser.add_argument(
        Action.START.option,
        action='store_true',
        help=Action.START.description
    )
    parser.add_argument(
        Action.SCAN.option,
        action='store_true',
        help=Action.SCAN.description
    )
    parser.add_argument(
        Action.SEARCH.option,
        type=str,
        metavar=Action.SEARCH.parameter,
        help=Action.SEARCH.description
    )
    parser.add_argument(
        Action.SUGGEST.option,
        action='store_true',
        help=Action.SUGGEST.description
    )
    parser.add_argument(
        Action.DOWNLOAD.option,
        type=str,
        metavar=Action.DOWNLOAD.parameter,
        help=Action.DOWNLOAD.description
    )
    parser.add_argument(
        Action.DELETE.option,
        type=str,
        metavar=Action.DELETE.parameter,
        help=Action.DELETE.description
    )

    args = parser.parse_args()

    try:
        if not mlx_hub.has_token():
            execute_action(Action.LOGIN)
        elif args.start:
            execute_action(Action.START)
        elif args.scan:
            execute_action(Action.SCAN)
        elif args.search:
            execute_action(Action.SEARCH, args.search)
        elif args.suggest:
            execute_action(Action.SUGGEST)
        elif args.download:
            execute_action(Action.DOWNLOAD, args.download)
        elif args.delete:
            execute_action(Action.DELETE, args.delete)
        else:
            parser.print_help()

    except Exception as e:
        print(f"An error occurred: {e}")


if __name__ == "__main__":
    main()
